#include <Arduino.h>

#define testbit(data,bitno) ((data>>bitno)&0x001)    //Byte manipulation macros
#define bit_set(var,bitno) ((var) |= 1 << (bitno))
#define bit_clr(var,bitno) ((var) &= ~(1 << (bitno)))

#define LED     13
//      BT RX   0
//      BT TX   1
#define latch   2
#define clk     3
#define data    4
#define HOME    5
#define Vbatt   A0
#define RED     9
#define GREEN   10
#define BLUE    11
#define paired  6


uint8_t serial_byte0 = 0;
uint8_t serial_byte1 = 0;

uint16_t value = 0;
uint8_t  batt_max = 800;
uint8_t  batt_cnt = 0;
double Vbattery = 0;
const double Vbatt_scale = 3.3*2; //0.5x voltage divider on adc pin

double vlow  = 3.4;
double low = 3.6;
double mid = 3.9;
//high = 4.2;

bool dpad_lt = 0;
bool dpad_rt = 0;
bool dpad_up = 0;
bool dpad_dn = 0;

bool face_a = 0;
bool face_b = 0;
bool face_x = 0;
bool face_y = 0;

bool sh_lt = 0;
bool sh_rt = 0;

bool start = 0;
bool select = 0;
bool _home = 0;

void snes_serial(void){
  //PORTB |= 0b00100000; //digitalWrite(LED,HIGH); //check how long it takes to read all buttons!!!!
  value=0xFFFF;
  PORTD |= 0b00000100; //digitalWrite(latch,HIGH);
  delay(2);
  PORTD &= 0b11111011; //digitalWrite(latch,LOW);
  for(unsigned int i=0;i<16;i++){
    if(digitalRead(data)==0){
      bit_clr(value,15-i);
    }
    delay(1);
    PORTD &= 0b11110111; //digitalWrite(clk,LOW);
    delay(1);
    PORTD |= 0b00001000; //digitalWrite(clk,HIGH);
  }
  //PORTB &= 0b11011111; //digitalWrite(LED,LOW);
} 

void setup() {
  pinMode(LED,OUTPUT);
  
  pinMode(latch,OUTPUT);
  pinMode(clk,OUTPUT);
  pinMode(data,INPUT);

  pinMode(HOME,INPUT_PULLUP);
//  pinMode(Vbatt,INPUT);
//  pinMode(RED,OUTPUT);
//  pinMode(GREEN,OUTPUT);
//  pinMode(BLUE,OUTPUT);

  pinMode(paired,INPUT_PULLUP);

  digitalWrite(clk,HIGH);
  
  Serial.begin(38400);//115200);

  snes_serial();
  if(!testbit(value,13)){  //if select held at startup, cycle through colors
    analogWrite(RED,170);   //253 Purple
    analogWrite(GREEN,255); //255
    analogWrite(BLUE,210);  //250
    delay(2000);
    analogWrite(RED,255);   //255 Blue
    analogWrite(GREEN,255); //255
    analogWrite(BLUE,205);  //245
    delay(2000);
    analogWrite(RED,255);   //255 Green
    analogWrite(GREEN,190); //250
    analogWrite(BLUE,255);  //255
    delay(2000);
    analogWrite(RED,170);   //250 Orange
    analogWrite(GREEN,212); //252
    analogWrite(BLUE,255);  //255
    delay(2000);
    analogWrite(RED,170);   //250 Red
    analogWrite(GREEN,255); //255
    analogWrite(BLUE,255);  //255
    delay(2000);
  }
}

void loop() {
  snes_serial();
  // SNES
  // 15 14 13 12 11 10 9  8  7 6 5  4  3 2 1 0
  // B  Y  Se St U  D  L  R  A X Le Ri x x x x
  //
  // NES
  // 7 6 5  4  3 2 1 0
  // A B Se St U D L R
  dpad_lt = !testbit(value,9);
  dpad_rt = !testbit(value,8);
  dpad_up = !testbit(value,11);
  dpad_dn = !testbit(value,10);

  face_a = !testbit(value,7);
  face_b = !testbit(value,15);
  face_x = !testbit(value,6);
  face_y = !testbit(value,14);

  sh_lt = !testbit(value,5);
  sh_rt = !testbit(value,4);

  start = !testbit(value,12);
  select = !testbit(value,13);
  _home = !digitalRead(HOME);

  serial_byte0 = (0<<7)|(dpad_dn<<6)|(dpad_lt<<5)|(dpad_rt<<4)|(face_a<<3)|(face_b<<2)|(face_x<<1)|(face_y);
  serial_byte1 = (1<<7)|(_home<<5)|(sh_lt<<4)|(sh_rt<<3)|(start<<2)|(select<<1)|(dpad_up);
  Serial.write(serial_byte0);
  Serial.write(serial_byte1);

  if(dpad_lt||dpad_rt||dpad_up||dpad_dn||face_a||face_b||face_x||face_y||sh_lt||sh_rt||start||select||_home){
    PORTB |= 0b00100000; //    digitalWrite(LED,HIGH);
  }else{
    PORTB &= 0b11011111; //    digitalWrite(LED,LOW);
  }

  if(batt_cnt<batt_max){
    batt_cnt++;
  }else{
    //digitalWrite(LED,HIGH); //CHECK THIS PERIOD AND EXECUTION TIME
    batt_cnt=0;
    Vbattery = (Vbatt_scale*analogRead(Vbatt)/1023);

    if(!digitalRead(paired)){             //Not paired, purple
      analogWrite(RED,170);   //253
      analogWrite(GREEN,255); //255
      analogWrite(BLUE,210);  //250
    }else if(Vbattery<vlow){                   //Very low battery, red
      analogWrite(RED,170);   //250
      analogWrite(GREEN,255); //255
      analogWrite(BLUE,255);  //255
    }else if(Vbattery>=vlow && Vbattery<low){  //Low battery, orange
      analogWrite(RED,170);   //250
      analogWrite(GREEN,212); //252
      analogWrite(BLUE,255);  //255
    }else if(Vbattery>=low && Vbattery<mid){  //Medium battery, green
      analogWrite(RED,255);   //255
      analogWrite(GREEN,190); //250
      analogWrite(BLUE,255);  //255
    }else if(Vbattery>=mid){                   //High battery, blue
      analogWrite(RED,255);   //255
      analogWrite(GREEN,255); //255
      analogWrite(BLUE,205);  //245
    }
    //digitalWrite(LED,LOW);
  }

/*    while(1){
      Vbattery=5*analogRead(Vbatt)/1023;
      Serial.println(Vbattery);
    }*/
}
